/**************************************************************************  P_LED.C
* Steuerung eines LED Clusters with RC5 remote control by Peter Dannegger
* Lothar Lehmann  12/08			Last Update 14.06.09
************************************************************************************/

#include	"P_LED.h"
#include	"rc5.h"

uint16_t				i;
uint16_t	volatile	I_ist;		// LED Current
uint16_t	volatile	I_soll;		// LED Current
uint8_t 				Ramp_en;
uint16_t				Ramp;
uint16_t				i_old;	

uint16_t const I_led[] =			// log. Steps
{20, 30, 45, 68, 102, 153, 230, 345, 520, 800};

uint16_t readADC(void)
{
    while (ADCSRA & (1<<ADSC));		// wait to finished
    ADCSRA |= (1<<ADSC);			// start single conversion
    return ADCW;
}

int	main(void)
{									// *********************** IO
  I_O	= (1<<PWM)|(1<<2);			// PB2 Check
  DDRB	= (1<<PWM)|(1<<2);
  I_O	&= ~(1<<PWM);				// FET off

  xRC5_DDR &= ~(1<<xRC5);       	// Pin = input
  xRC5_OUT |= (1<<xRC5);      		// Pin pullup
									// *********************** ADC
  ADCSRA =(1<<ADEN)|(1<<ADPS2)|(1<<ADPS1);
  DIDR0	= (1<<ADC2D)|(1<<ADC3D);	// ADC2 and ADC3 as input
  ADMUX	= Mux;
  ACSR	= 1<<ACD;					// analog Comp. disable
  ADCSRA |= 1<<ADSC;				// Start single conversion
  while (ADCSRA & (1<<ADSC));		// wait to finished
									// *********************** TC0
  TCCR0B= 1<<CS02;					// divide by 256
  TIMSK	= 1<<TOIE0;					// enable Timer Int.
									// *********************** TC1
  TCCR1	= (1<<PWM1A)|(1<<COM1A1)|(1<<CS10);
  PLLCSR = (1<<LSM)|(1<<PCKE)|(1<<PLLE);
  OCR1A	= 0x05;						// PWM = 5  2%
  while (PLLCSR & (1<<PLOCK));		// wait for PLL is locked

  sei();							// enable Int. global
  I_soll = 131;						// equal key 5
  Ramp_en = true;
  while(1)
  {
  	cli();
    i_old = i;
	i = rc5_data;					// read two bytes from interrupt 
	rc5_data = 0;
	sei();
 	if(i)
	{
	  i &= 0x3F;					// code only
	  if (i == 12) 
	  {
	    Ramp_en = true;
	    while (I_ist > 20)			// device off
		{ 
		  OCR1A--;
		  while (Ramp < 300);
		  Ramp = 0;
		  I_ist = readADC();
		}
	    TCCR1 &= ~(1<<PWM1A);	 	// PWM off
		OCR1A = 0x05;
		I_soll = I_led[0];
		Ramp_en = true;
	  } 
	  else {TCCR1 |= (1<<PWM1A);}	// PWM on
	  if (i<10) {I_soll=I_led[i];}	// key 0..9
	  else
	  {
	    switch(i)
	    {
	      case	16:					// Volume +
		    I_soll += (I_soll>>4);	// Step variable
		    if (I_soll>I_led[9]) I_soll = I_led[9];
	        break;
	      case	17:					// Volume -
		    I_soll -= (I_soll>>4);
	        if (I_soll<I_led[0]) I_soll = I_led[1];
	        break;
        }
      }
    }
    I_ist = readADC();
	if (!(i == i_old)) 
	  { Ramp = 0;
	    Ramp_en = true;
       }
	if (Ramp_en)					// DimmStep at 50ms
	{
	  while (Ramp < 780);
	  Ramp = 0;
	}
	if (I_ist < I_soll) {OCR1A++;}	// increment PWM
	if (I_ist > I_soll) {OCR1A--;}	// decrement PWM
	if (abs(I_ist - I_soll) < 10) {Ramp_en = false;}
	
  }
}
